unit hiThread;

interface

{$I share.inc}

uses Kol, Share, Windows, Debug, NSThreads;

type
  THIThread = class(TDebug)
    private
      th: TThread;

      function Execute(Sender: TCustomThread): Integer;
      procedure SyncExec(Param: Pointer);
      
    public
      _prop_Delay: Cardinal;
      _prop_FastStop: Boolean;
      _prop_BusyEvent: Byte;

      _event_onSyncExec: THI_Event;
      _event_onExec: THI_Event;

      destructor Destroy; override;
      procedure _work_doStart(var _Data: TData; Index: Word);
      procedure _work_doStop(var _Data: TData; Index: Word);
      procedure _work_doStopFlag(var _Data: TData; Index: Word);
      procedure _work_doDelay(var _Data: TData; Index: Word);
      procedure _var_Busy(var _Data: TData; Index: Word);
      procedure _var_Handle(var _Data: TData; Index: Word);
  end;

implementation

destructor THIThread.Destroy;
begin
  _event_onExec.Event := nil;
  _event_onSyncExec.Event := nil;
  
  if th <> nil then
  begin
    th.Stop;
    th.DecRef;
  end;
  
  inherited;
end;

function THIThread.Execute(Sender: TCustomThread): Integer;
begin
  Result := 0;
  
  repeat
    Sleep(_prop_Delay);
    if Sender.Stopped then Break;
    
    _hi_OnEvent(_event_onExec);
    if Assigned(_event_onSyncExec.Event) then Sender.Synchronize(SyncExec);
  
  until Sender.Stopped or _prop_FastStop;
end;

procedure THIThread.SyncExec(Param: Pointer);
begin
  _hi_OnEvent(_event_onSyncExec);
end;

procedure THIThread._work_doStart;
begin
  if Assigned(th) then
  begin
    if (_prop_BusyEvent = 0) and (not th.Terminated) then Exit;
    th.Stop;
    th.DecRef;
  end;

  th := TThread.Create;
  th.OnExecute := Execute;
  th.Resume;
end;

procedure THIThread._work_doStop;
begin
  if th <> nil then
  begin
    th.Terminate;
    th.DecRef;
    th := nil;
  end;
end;

procedure THIThread._work_doStopFlag;
begin
  if Assigned(th) then th.Stop;
end;

procedure THIThread._work_doDelay;
begin
  _prop_Delay := ToInteger(_Data);
end;

procedure THIThread._var_Busy;
begin
  dtInteger(_Data, Integer(Assigned(th) and (not th.Terminated)));
end;

procedure THIThread._var_Handle;
begin
  if th <> nil then
    dtInteger(_Data, th.ThreadHandle)
  else
    dtInteger(_Data, 0);
end;

end.
